'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createSource = createSource;

var _union = require('lodash/union');

var _union2 = _interopRequireDefault(_union);

var _difference = require('lodash/difference');

var _difference2 = _interopRequireDefault(_difference);

var _createSource2 = require('./createSource');

var _createSource3 = _interopRequireDefault(_createSource2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var combineMethods = {
  replace: function replace(_ref) {
    var result = _ref.result,
        entities = _ref.entities;

    return {
      result: result,
      entities: entities
    };
  },
  merge: function merge(_ref2) {
    var prevResult = _ref2.prevResult,
        prevEntities = _ref2.prevEntities,
        result = _ref2.result,
        entities = _ref2.entities;

    var newState = {};
    var newResults = _extends({}, prevResult);
    for (var resultName in result) {
      if (Array.isArray(result[resultName]) && (result[resultName].length === 0 || _typeof(result[resultName][0]) !== 'object')) {
        newResults[resultName] = (0, _union2.default)(prevResult[resultName], result[resultName]);
      } else {
        newResults[resultName] = result[resultName];
      }
    }
    var newEntities = {};
    for (var entityName in entities) {
      newEntities[entityName] = _extends({}, prevEntities[entityName] || {});
      for (var entityId in entities[entityName]) {
        newEntities[entityName][entityId] = _extends({}, (prevEntities[entityName] || {})[entityId] || {}, entities[entityName][entityId]);
      }
    }
    if (Object.keys(result).length) {
      newState.result = _extends({}, prevResult, newResults);
    }
    if (Object.keys(newEntities).length) {
      newState.entities = _extends({}, prevEntities, newEntities);
    }
    return newState;
  },
  crop: function crop(_ref3) {
    var prevResult = _ref3.prevResult,
        result = _ref3.result;

    var newState = {};
    var newResult = _extends({}, prevResult);
    var isResultChange = false;
    for (var resultName in result) {
      if (Array.isArray(result[resultName]) && (result[resultName].length === 0 || _typeof(result[resultName][0]) !== 'object')) {
        var filterd = (0, _difference2.default)(prevResult[resultName], result[resultName]);
        if (filterd.length !== prevResult[resultName].length) {
          newResult[resultName] = filterd;
          isResultChange = true;
        }
      }
    }
    if (isResultChange) {
      newState.result = newResult;
    }
    return newState;
  }
};

function createSource(_ref4) {
  var typeDefs = _ref4.schema,
      resolvers = _ref4.resolvers;

  return (0, _createSource3.default)({
    typeDefs: typeDefs,
    resolvers: resolvers,
    createInitialState: function createInitialState(_ref5) {
      var stateName = _ref5.stateName;
      return _defineProperty({}, stateName, {
        result: {},
        entities: {},
        isPending: false,
        pendingCount: 0,
        errors: []
      });
    },
    pendingReducer: function pendingReducer(_ref7) {
      var stateName = _ref7.stateName;
      return function (state) {
        var newSourceState = {
          isPending: true,
          pendingCount: state[stateName].pendingCount + 1
        };
        if (state[stateName].errors.length) {
          newSourceState.errors = [];
        }
        return _extends({}, state, _defineProperty({}, stateName, _extends({}, state[stateName], newSourceState)));
      };
    },
    successReducer: function successReducer(_ref8) {
      var stateName = _ref8.stateName,
          config = _ref8.config;
      return function (state, _ref9) {
        var _ref9$payload = _ref9.payload,
            payload = _ref9$payload === undefined ? {} : _ref9$payload;
        var _state$stateName = state[stateName],
            prevResult = _state$stateName.result,
            prevEntities = _state$stateName.entities,
            pendingCount = _state$stateName.pendingCount;

        var combineMethod = combineMethods[config.combineResult || 'merge'];
        var newNormalizedData = combineMethod({
          prevResult: prevResult,
          prevEntities: prevEntities,
          result: payload.result || {},
          entities: payload.entities || {}
        }) || {};
        var count = pendingCount - 1;
        var newSourceState = _extends({}, newNormalizedData, {
          isPending: count > 0,
          pendingCount: count
        });
        if (state[stateName].errors.length) {
          newSourceState.errors = [];
        }
        return _extends({}, state, _defineProperty({}, stateName, _extends({}, state[stateName], newSourceState)));
      };
    },
    errorReducer: function errorReducer(_ref10) {
      var stateName = _ref10.stateName;
      return function (state, _ref11) {
        var payload = _ref11.payload;

        var count = state[stateName].pendingCount - 1;
        return _extends({}, state, _defineProperty({}, stateName, _extends({}, state[stateName], {
          isPending: count > 0,
          pendingCount: count,
          errors: payload
        })));
      };
    }
  });
}
//# sourceMappingURL=index.js.map